/**!
 * Busca coincidencias de nombres de congresistas y ex-congresistas en un artículo de elespectador.com
 *
 * @author John Acosta<john@monoku.com> / updated by: David Avellaneda<david@monoku.com>
 * Copyright (C) 2011 monoku S.A.S
 */
(function( $ ){
	var 
		/**
		 * URL de Congreso Visible
		 * @type String
		 */
		cvisibleurl = "http://www.congresovisible.org/",

		/**
		 * URL de archivos estáticos en S3 desde donde se cargaran fotos de congresistas y demás info
		 * @type String
		 */
		cvisibleS3 = "http://static.cv.elespectador.com/",

		/**
		 * Set de URLs que se utilizarán para cargar recursos
		 * @type Object
		 */
		urls = {
			collection: cvisibleS3 + "media/js/elespectador/js_names.js",
			profile: cvisibleurl + "congresistas/perfil/{name}/{id}/",
			loading: cvisibleS3 + "media/img/elespectador/loading.gif",
			spacegif: cvisibleS3 + "media/img/elespectador/espacio.gif",
			backright: cvisibleS3 + "media/img/elespectador/back-right.png",
			logo: cvisibleS3 + "media/img/elespectador/cvisible.jpg",
			tab: cvisibleS3 + "media/img/elespectador/cvisible-tab.jpg",
			close: cvisibleS3 + "media/img/elespectador/close.jpg",
			borderright: cvisibleS3 + "media/img/elespectador/border-right.jpg",
			plus: cvisibleS3 + "media/img/elespectador/masroja.png"
		},
		
		/**
		 * Objeto jQuery que contiene los nodos donde se buscará texto para la búsqueda
		 * @type jQuery
		 */
		nodes = $('.sumario h3,.content_nota > *'),

		/**
		 * Referencia al nodo del tooltip
		 * @type HTMLElement
		 */
		tooltip = null,

		/**
		 * Referencia al nodo del tab
		 * @type HTMLElement
		 */
		tabContainer = null,

		/**
		 * Referencia al nodo del elemento que abre el tab, en este caso un anchor
		 * @type HTMLElement
		 */
		tabTrigger = null,

		/**
		 * Arreglo de tokens de texto sacados de los nodos de búsqueda
		 * @type Array
		 */
		elements = []
	;
		
	// Realiza la solicitud al servidor
	request();
	
	/**
	 * Realiza al solicitud de los nombres de los congresistas al servidor
	 * @returns {Boolean} true
	 */
	function request(){
		loadScript(urls.collection, process);
		return true;
	}
	
	/**
	 * Crea la estrutura DOM del tooltip y la inyecta en el documento
	 */	
	function createTooltip(){
		var 
			container = document.createElement('div'),
			loading = document.createElement('div'),
			placeholder = document.createElement('img')
		;
		
		$(container).attr('id', 'cvisible-tooltip-info').css({
			position: 'absolute',
			display: 'none'
		});
		
		$(placeholder).attr('class', 'placetp').css('cursor', 'pointer').attr({
			src: urls.spacegif,
			width: '80px',
			height: '300px',
			'class': 'placetp'
		});
		
		$(loading).attr('class', 'loading').css({
			background: '#FFF url('+ urls.loading +') center center no-repeat',
			dth: '264px',
			height: '350px'
		});
		
		container.appendChild(placeholder);
		container.appendChild(loading);
		
		document.body.appendChild(container);
	}

	/**
	 * Crea la estrutura DOM del tab y la inyecta en el documento
	 */
	function createTab(){
		var tab = document.createElement('div'),
			trigger = document.createElement('a'),
			info = document.createElement('div'),
			inner = document.createElement('div'),
			itemList = document.createElement('ul'),
			footer = document.createElement('div'),
			iframe = null,
			close = document.createElement('a');
		
		$(tab).attr('id', 'cvisible-tab').css({
			position: 'fixed',
			width: '607px',
			height: '291px',
			left: '-638px',
			top: '50%',
			marginTop: '-150px',
			height: '295px',
			border: 'solid 1px #CCC',
			zIndex: 9999
		});
		
		$(trigger).attr({href: '#', id: 'cvisible-tab-trigger'}).css({
			background: "url("+ urls.tab +") no-repeat",
			position: 'fixed',
			width: '35px',
			height: '149px',
			left: '0px',
			top: '50%',
			marginTop: '-76px'
		}).click(function(){
			$(tab).animate({left: '0px'});
			return false;
		});
		
		$(info).attr('id', 'cvisible-tab-info').css({
			'float': 'left',
			width: '190px',
			textAlign: 'left'
		})
		
		$(inner).css({
			height: '238px',
			padding: '6px 9px',
			background: 'url('+ urls.backright +') repeat-x'
		}).html('<h3>&nbsp;</h3><p>En este artículo también encontrará a</p>');
		

		$(inner).find('h3').css({
			fontSize: '12px', 
			fontFamily: 'Arial,Helvetica,utkal,sans-serif',
			background: 'url('+ urls.borderright +') center bottom no-repeat',
			color: '#FFF',
			paddingBottom: '6px',
			marginBottom: '5px',
			textAlign: 'left',
			fontWeight: 'bold'
		});
		
		$(inner).find('p').css({
			fontSize: '12px', 
			fontFamily: 'Arial,Helvetica,utkal,sans-serif',
			color: '#FFF',
			fontWeight: 'bold',
			marginBottom: '5px'
		});
		
		$(itemList).attr('id', 'cvisible-itemlist').css({'overflow': 'auto', 'max-height': '177px', 'margin-right': '-8px'});
		
		$(footer).css({
			background: '#FFF url('+ urls.logo +') no-repeat',
			height: '45px'
		});
		
		iframe = $('<iframe frameborder="0" name="cvisible-tag-iframe"></iframe>').attr({
			id: 'cvisible-tag-iframe',
			width: '417',
			height: '295',
			allowtransparency: 'off'
		}).css({
			'float': 'left',
			backgroundColor: '#F2F2F2'
		})[0];
		
		$(close).attr({
			id: 'cvisible-closetab',
			href: '#'
		}).css({
			background: 'url('+ urls.close +') no-repeat',
			width: '29px',
			height: '65px',
			position: 'absolute',
			top: 0,
			right: '-29px'
		}).click(function(){
			$(tab).animate({left: '-638px'}); // cuando se cierra el tab
			return false;
		});
		
		inner.appendChild(itemList);
		
		info.appendChild(inner);
		info.appendChild(footer);
		
		tab.appendChild(close);
		tab.appendChild(iframe);
		tab.appendChild(info);
		
		//exportando al scope superior
		tabTrigger = trigger;
		tabContainer = tab;
	}
	
	/**
	 * Toma un objeto jQuery, lo recorre, extrae el texto, le quita el formato y lo devuelve como cadena
	 *
	 * @param {jQuery} els Objeto jQuery de elemento
	 * @returns {String} Texto sin formato de los nodos dentro del objeto jQuery
	 */
	function createTextFrom(els){
		var expr = /<(\w+)[^>]+>|<\/\\1[^>]+>/gi;
		var string = '';
		
		for(var i = 0, l = els.length; i < l; i++){
			string += els[i].data.replace(expr, '');
		}
		
		return string;
	}
	
	/**
	 * Procesa los nodos de búsqueda y reemplaza las coincidencias con triggers del tooltip
	 */
	function process(){
		// extrae los nodos de texto en elements[]
		traverse(); 
		// crea el tooltip
		createTooltip();
		// crea el tab
		createTab();
		
		var mtch, anchor, text, item, n1, nextIndex;
		var names = window.nombres; // creamos una referencia local al objeto de nombres que viene del servidor
		var ids = window.ids; // creamos una referencia local al objeto de nombres que viene del servidor
		var anchorBase = document.createElement('a'); // Un anchor que se clonará para ser insertado en vez de cada coincidencia
		var range = new CustomRange(); // Instancia al Wrapper de TextRange
		
		text = createTextFrom(elements); // creamos el texto apartir de los nodos de búsqueda
		text = stripNL( pseudoSlugify(text) ); // IE tiene problemas con los saltos de línea y retornos de carro en el código fuente :S
		
		var iframeLoaded = false;
		var showTooltip = false; // indica si el tooltip está visible
		
		
		// Evento click que oculta el tab
		$(document.body).click(function(e){
			if( !$(e.target).hasClass('cvisible-poptrigger') 
					&& $(e.target) !== tabContainer 
					&& !$(e.target).hasClass('placetp') 
					&& !$(e.target).parents('#cvisible-tab').length){
				$(tabContainer).animate({left: '-630px'});
			}
		});
		
		// Evento para que si el mouse se para sobre el tooltip, éste no se cierre
		$('#cvisible-tooltip-info').hover(function(){showTooltip = true;}, function(){
			setTimeout(function(){ // cuando se salga del tooltip, se cierra 400ms después a menos que vuelta a entrar
				if( !showTooltip ){
					$('#cvisible-tooltip-info').css('display', 'none');
				}
			}, 400);
			showTooltip = false;
		});
		
		var idTo = 0; // congresista a mostrar
		
		// se crea un placeholder con clase placetp para corregir un fix de capas en IE
		// se dispara cuando el usuario hace click en Ver Más y muestra el tab
		$('#cvisible-tooltip-info').find('.placetp').click(function(){
			$(tabContainer).animate({left: 0}); // se muestra el tab
			var href = $('#cvisible-itemlist').find('li.cong-'+ idTo + ' a').attr('href');
			// cambia la ubicación del iframe del tab para mostrar la info del congresista
			$('#cvisible-tag-iframe')[0].contentWindow.location.href = href ; 
			$('#cvisible-tooltip-info').css('display', 'none').find('.placetp').css('display', 'none');//oculta el tooltip
			return false;
		}).css({ // posiciona el placeholder dentro del tooltip
			position: "absolute",
			right: '15px',
			bottom: '30px',
			width: '80px',
			height: '300px',
			lineHeight: '30px',
			zIndex: 99,
			display: 'none',
			textIndent: '-9999px',
			zoom: 1
		});
		
		var putd = 0;// cuenta la cantidad de congresistas que se han encontrado en el texto
		var lastId = 0; // id del último congresista encontrado
		
		// captura los elementos <br /> dentro de los nodos de busqueda
		var brs = nodes.find('br');
		
		// se recorre el listado de nombres que llegan desde el servidor
		for(var i = 0, l = names.length; i < l ; i++){
			name = names[i]; // guardamos el texto del nombre
			// representa el offset de busqueda dentro del texto
			// cuando se encuentra un nombre, se vuelve a empezar desde el indice final del nombre
			nextIndex = 0;
			
			// realizando la busqueda
			while( (mtch = search(name, text, nextIndex)) >= 0 ){
				//Aquí falla en IE9
				n1 = getElementByIndex( mtch, name, text ); // Buscamos el nodo DOM y la posición de este donde se encuentra el texto
				
				if( !n1 ){continue;}
				
				nextIndex = mtch + name.length; // recalcula el siguiente offset para buscar
				
				// no avanza si el nodo es un anchor para evitar insertar anchor en anchor
				if( n1.node.parentNode && n1.node.parentNode.tagName === 'A' ){continue;}
				
				// si el nombre encontrado es distinto al encontrado en la anterior iteracion...
				// verifica si se han puesto al menos 2 congresistas para mostrar el nodo que muestra el tab
				if(lastId !== ids[i]){
					putd++;
					
					if( putd === 2){
						document.body.appendChild(tabTrigger);
					}
					
					document.body.appendChild(tabContainer);
					lastId = ids[i]; // actualiza el último ID
				}
				
				// Establecemos los limites al rango de texto a reemplazar
				// Le decimos el nodo y la posición inicial + el nodo y la posición final
				range.setBoundary({
					node: n1.node, 
					pos: (mtch - n1.pos) 
				},{
					node: (n1.nodeEnd || n1.node),
					pos: n1.posEnd || ((mtch - n1.pos) + name.length)
				});
				
				// se crea el anchor que se va a insertar
				anchor = anchorBase.cloneNode(false);
				// se configura el anchor para que tenga el texto del nombre
				// y lo inserta en el rango de texto
				anchor = buildLink(range, anchor);
				
				// Se adjuntan los eventos mouseout y mouseover
				$(anchor).addClass('cvisible-poptrigger').hover((function(id){
					return function(e){	
						// se crea el iframe para mostrar en el tooltip						
						var iframe = $('<iframe src="'+ cvisibleS3 +'media/elespectador/congresista/'+ id +'/index.html" frameborder="0" allowTransparency="1"></iframe>');
						// estilos para el tooltip
						iframe.css({width: '280px', height: '363px', border: '0', display: 'none'}).load(function(){ // evento load
							$('#cvisible-tooltip-info').find('.loading').hide(); // quita el gir de cargando
							$('#cvisible-tooltip-info').find('.placetp').show(); // muestra el contenido del tab
							$(this).css('display', 'block');
						});			
						
						idTo = id;
						
						$('#cvisible-tooltip-info').find('.placetp').css('display', 'none');//oculta el contenido del iframe
						$('#cvisible-tooltip-info').find('iframe').remove(); // quita el iframe anterior
						$('#cvisible-tooltip-info').append(iframe); // inyenta el nuevo iframe
						$('#cvisible-tooltip-info').find('.loading').show(); // muestra el gif de cargando
						 //mueve el tooltip a la posición del anchor y lo muestra
						$('#cvisible-tooltip-info').css({
							left: e.pageX - Math.round(260 / 2) + 'px',
							top: e.pageY - $('#cvisible-tooltip-info').height() - 20 + 'px',
							display: 'block',
							zIndex: 9999
						});
						
						showTooltip = true;// se cambia el flag
					};
				})(ids[i]), function(){ // acá es cuando se oculta el tooltip, es decir, cuando se está fuera del anchor
					setTimeout(function(){
						if( !showTooltip ){
							$('#cvisible-tooltip-info').css('display', 'none').find('.placetp').css('display', 'none');
						}
					}, 400);
					
					showTooltip = false;
				}).click(function(){
					var href = $('#cvisible-itemlist').find('li.cong-'+ idTo + ' a').attr('href');
					// cambia la ubicación del iframe del tab para mostrar la info del congresista
					$('#cvisible-tag-iframe')[0].contentWindow.location.href = href ; 
					$('#cvisible-tooltip-info').css('display', 'none').find('.placetp').css('display', 'none');//oculta el tooltip
					$(tabContainer).animate({left: '0px'}); // cuando se clickea sobre el link, se oculta el tab
					return false;
				});
				
				//se crea un span para agregar el + que decora
				var span = document.createElement('span'); 
				
				$(span).css({
					background: 'url('+ urls.plus +') right top no-repeat',
					display: 'none',
					width: '16px',
					height: '16px',
					marginLeft: '3px'
				}).attr('class', 'cv-span-deco');
				
				anchor.appendChild(span);

				// El Espectador por defecto le pone un fondo a los anchor...
				// ...nosotros se lo quitamos
				$(anchor).css({
					background: 'none',
					paddingRight: '2px',
					margin: 0
				});
				
				// Si no se ha agregado a la lista de congresistas del TAB, pues se agrega
				if( !$('#cvisible-itemlist').find('li.cong-'+ ids[i] ).length ){
					// Obsoleto
					if(!iframeLoaded){ // en realidad esto nunca sucede.
						$('#cvisible-tag-iframe').attr('src', cvisibleS3 + "media/elespectador/congresista/"+ ids[i] +"/large/index.html");
					}
					// acá se agrega a la lista del tab
					$('#cvisible-itemlist').append('<li class="cong-'+ ids[i] +'" style="float: left; margin-right: 8px; margin-bottom: 8px;"><a href="http://static.cv.elespectador.com/media/elespectador/congresista/'+ ids[i] +'/large/index.html" style="display: block; width: 46px; height: 46px; border: 0;" target="cvisible-tag-iframe"><img src="'+ cvisibleS3 +'media/elespectador/congresista/'+ ids[i] +'.jpeg" border="0" width="45" height="45" /></a></li>');
				}
				
				// reorganiza los nodos teniendo en cuenta que si tomamos un nodo de texto y le insertamos un anchor
				// ya no es 1 nodo, son 3 nodos
				var nodesRecruited = recruitNodes(anchor);
				anchor.nextSibling && nodesRecruited.push(anchor.nextSibling);
				// se actualiza el arreglo de nodos de texto general
				if($.browser.msie && (+$.browser.version === 9) ){
					elements.splice.apply(elements, [n1.index, 1].concat(nodesRecruited));
				}
				else
					elements.splice.apply(elements, [n1.index +1, 0].concat(nodesRecruited));
			}			
		}
		
		$('.cv-span-deco').css('display', 'inline-block');
	}
	
	/**
	 * Función recursiva que recorre un nodo y devuelve un arreglo con los nodos de texto internos
	 *
	 * @param {HTMLElement} el Elemento HTML que será recorrido
	 * @param {Array} [arr] Arreglo de nodos temporal
	 * @returns {Array}
	 */
	function recruitNodes(el, arr){
		arr = arr || [];
		
		for(var i = 0, l = el.childNodes.length; i < l; i++){ // recorre los hijos directos en busca de texto			
			if( el.childNodes[i].nodeType === 1 ){ // si son Nodos HTML se llama a si misma
				recruitNodes(el.childNodes[i], arr);
			}
			else if( el.childNodes[i].nodeType === 3 ){// si es un nodo de texto lo inyecta en el arreglo
				arr.push(el.childNodes[i]);
			}
		}
		
		return arr;
	}
	
	/**
	 * Busca un termino dentro de un texto y devuelve la ubicación
	 *
	 * @param {String} term Término a buscar
	 * @param {String} text Texto en el que se realizará la busqueda
	 * @param {Number} [offset] Indice desde donde se realizará la busqueda
	 * @returns {Number} Indice donde se encontró la primera coincidencia
	 */
	function search(term, text, offset){
		var index = text.indexOf(term, offset);
		return index;
	}
	
	/**
	 * Recorre los nodos de busqueda
	 * @see #traverseOne
	 */
	function traverse(){
		for(var i = 0, l = nodes.length; i < l; i++){
			traverseOne( nodes[i] );
		}
	}
	
	/**
	 * Recorre recursivamente un nodoo en busqueda de nodos de texto y los inyecta en el arreglo general, elements
	 * @param {HTMLElement} el Elemento a recorrer
	 */
	function traverseOne(el){			
		/**
		 * skipping anchor elements because you cannot insert an anchor inside another one 
		 */
		if( el && el.nodeType === 1 && el.childNodes.length && el.tagName !== 'A' ){
			for(var i = 0, l = el.childNodes.length; i < l; i++){
				traverseOne(el.childNodes[i]);
			}
		}
		
		//text node --> #text
		else if(el.nodeType === 3){				
			elements.push(el);
		}
	}
		
	/**
	 * Inserta un anchor en el rancho de texto pasado como parametro
	 *
	 * @param {CustomRange} el La instancia del rango de texto
	 * @param {HTMLElement} anchor La referencia al anchor
	 * @returns {HTMLElement} el anchor modificado
	 */
	function buildLink(range, anchor){
		anchor.setAttribute('href', '#');
		range.surroundContents( anchor, elements ); // reemplaza el contenido del rango con el anchor
		return range.insertNode( anchor );
	}
		
	/**
	 * Devuelve un descriptor con el nodo incial, nodo final, posición inicial y final  relativas de un termino en el texto de varios nodos
	 *
	 * @param {Number} index Indice global donde se encuentra el término en el texto
	 * @param {String} item Texto de la busqueda
	 * @param {String} text Texto completo
	 * @returns {HTMLElement} el anchor modificado
	 */
	function getElementByIndex(index, item, text){
		var sum = 0;
		var in1;
		var maxLength = index + item.length;
		var posEnd = null;
		var nodeEnd = null;
		//HERE IT FAILS
		// recorre los nodos de texto
		for(var i = 0, l = elements.length; i < l; i++){
			in1 = sum;
			try{
				sum += stripNL(elements[i].nodeValue).length;
			}catch(e){}
			
			if( index < sum ){				
				if( maxLength > sum ){
					nodeEnd = elements[i + 1];
					posEnd = maxLength - sum;
				}
				return {node: elements[i], pos: in1, posEnd: posEnd, nodeEnd: nodeEnd, index: i};
			}
		}	
		
		return null;
	}
	

	/**
	 * Crea un wrapper alrededor del objeto TextRange para ofrecer compatibilidad entre IE y W3C DOM
	 *
	 * @constructor
	 */
	function CustomRange(){
		var range = null; // referencia al rango nativo
		var self = this;
		var mode = mode = { // verifica en qué funcionalidad tenemos disponible
			ie: ('createTextRange' in document.body),
			w3c: ('createRange' in document)
		};
		
		/**
		 * Devuelve la posición de un nodo en su nodo padre en caracteres
		 * @param {Text} node Nodo de texto a buscar
		 * @param {HTMLElement} node Elemento a buscar
		 * @returns {Number} posición del nodo dentro del padre
		 * @function
		 */
		this.seekNode = function(node, parentNode){		
			if( parentNode  ){
				for(var i = 0, l = parentNode.childNodes.length; i < l; i++){
					if( parentNode.childNodes[i].nodeType === 3 ){ // si es elemento actual es de texto
						if( parentNode.childNodes[i] === node ){ // y es el que estamos buscando, devuelve la posición actual
							return this.seekNode.$pos;
						}
						else{ // si no es el nodo actual, quita los espacios de linea y suma la posición y la longitud
							this.seekNode.$pos += stripNL(parentNode.childNodes[i].nodeValue).length;
						}
					}				
					// si es un Element HTML ejecuta revursivamente buscando más nodos de texto
					else if( parentNode.childNodes[i].nodeType === 1 && parentNode.childNodes[i].childNodes.length ){
						this.seekNode(node, parentNode.childNodes[i]);
					}
				}
			}
		};
		
		/**
		 * Reinicia la posición a cero
		 * @function
		 */
		this.seekNode.clear = function(){
			self.seekNode.$pos = 0;
		};
		
		/**
		 * Devuelve el contenido del rango como cadena
		 * @returns {String}
		 * @function
		 */
		this.toString = (function(){
			return mode.w3c ? function(){return range.toString();} :  function(){return range.text;};
		})();
		
		/**
		 * Cambia los límites del rango
		 *
		 * @param {Object} ini Descriptor con dos propiedades: node => nodo inicial, pos => posición dentro del nodo inicial
		 * @param {Object} end Descriptor con dos propiedades: node => nodo final, pos => posición dentro del nodo final
		 * @function
		 */
		this.setBoundary = (function(){
			return mode.w3c /*&& !(jQuery.browser.msie && (+jQuery.browser.version === 9))*/? 
					function(ini, end){ // W3C DOM
						range.setStart( ini.node, ini.pos );
						//Fix for bad editing
						end.pos = end.pos > end.node.textContent.length ? end.node.textContent.length-1: end.pos;
						range.setEnd( end.node, end.pos );
					} 
				: 
					function(ini, end, ed){ // IE
						// creamos dos rangos de texto nuevos
						var r2 = document.body.createTextRange();
						var r3 = document.body.createTextRange(); 
						var bini = ini.node, 
							bend = end.node,
							pini = ini.pos,
							pend = end.pos;
						
						// TextRange en IE no acepta nodos de texto
						// así que debemos pasarle el nodo padre de éste
						if(ini.node.nodeType === 3){
							this.seekNode.clear();
							ini.pos += this.seekNode(ini.node, ini.node.parentNode);
							ini.node = ini.node.parentNode;
						}
						
						if(end.node.nodeType === 3){
							this.seekNode.clear();
							end.pos += this.seekNode(end.node, end.node.parentNode);
							end.node = end.node.parentNode;
						}

						if(~$(end.node).parents().index(ini.node)){
							this.seekNode.clear();
							end.pos += this.seekNode(bend, ini.node);
							end.node = ini.node;
						}else if(~$(ini.node).parents().index(end.node)){
							this.seekNode.clear();
							ini.pos += this.seekNode(bini, end.node);
							ini.node = end.node;
						}
						
						// Movemos los limites del rango 3 al nodo incial
						// IE no permite especificar nodo y posición, solamente nodo
						r3.moveToElementText( ini.node );
						// Movemos el límite final del rango 3 la posición 0 del nodo inicial
						r3.collapse();
						// Movemos el limite final del rango a la posición incial dentro del nodo
						r3.moveEnd('character', ini.pos);
						
						// Contamos la cantidad de saltos de línea para luego sumarlos
						var match = r3.text.match(/[\n]/g);
						var nl = match ? match.length : 0;
						
						// movemos el rango principal al nodo inicial
						range.moveToElementText( ini.node );
						// movemos el limite inicial a la posición inicial + la cantidad de espacios de línea
						range.moveStart('character', ini.pos + nl);
						// movemos los límites del segundo rango al nodo final
						r2.moveToElementText( end.node );
						// movemos el limite inicial del segundo rango a la posición inicial +'los saltos de línea'
						r2.moveStart('character', ini.pos + nl);
						// movemos el limite final del rango principal al límite inicial del rango 2
						range.setEndPoint('EndToStart', r2);
						// movemos el limite final del rango principal al final del termino de busqueda
						range.moveEnd('character', ((end.pos - ini.pos)) );
					};
		})();
		
		/**
		 * Cambia el contenido del nodo especificado por el del rango
		 *
		 * @param {HTMLElement} node Elemento DOM a insertar el contenido
		 * @function
		 */
		this.surroundContents = (function( node ){
			return mode.w3c ? 
				function(node,els){
					var inner = range.extractContents();
					node.appendChild( inner); // extrae el contenido del Rango y lo inserta dentro del nodo
				}
			:
				function(node){
					// para IE, creamos un nodo de texto con el contenido del rango y luego borramos dicho contenido
					node.appendChild( document.createTextNode(range.text) );
					range.text = '';
				};
		})();
		
		/**
		 * Inserta el nodo dentro del rango
		 *
		 * @param {HTMLElement} node Elemento DOM a insertar
		 * @function
		 */
		this.insertNode = (function(){
			return mode.w3c ?
				function(node){
					range.insertNode(node);
					return node;
				}
			:
				function(node){
					var nodeId = '';
				
					do{
						nodeId = 'rnganchor-'+ (Math.random() * 999);
					}while( document.getElementById(nodeId) );
					 
					node.setAttribute('id', nodeId);
					
					var tmp = document.createElement('div');
					tmp.appendChild( node );
					
					range.pasteHTML( tmp.innerHTML );
					tmp.removeNode();
					
					return document.getElementById( nodeId );
				};
		})();
		
		/**
		 * Libera el rango de memoria
		 * @function
		 */
		this.detach = function(){
			if( mode.w3c ){
				range.detach();
			}
			else{
				//IE doesn't implement a way to release the range. IE Sux
				range = null;
			}
		};
		
		/**
		 * Selecciona el contenido del rango en el documento. Solamente IE.
		 * @function
		 */
		this.select = function(){
			if( mode.ie ){
				range.select();
			}
		};
		
		/**
		 * Mueve el contenido del rango a un elemento de texto
		 * @function
		 */
		this.moveToElementText = function(node){
			if( mode.ie ){
				range.moveToElementText(node);
			}
		};
		
		// creamos el TextRange
		range = mode.w3c ? document.createRange() : document.body.createTextRange();
	}
	
	/**
	 * by NCZ http://www.nczonline.net/blog/2009/06/23/loading-javascript-without-blocking/
	 */
	 function loadScript(url, callback){

	    var script = document.createElement("script");
	    script.type = "text/javascript";

	    if (script.readyState){  //IE
	        script.onreadystatechange = function(){
	            if (script.readyState == "loaded" ||
	                    script.readyState == "complete"){
	                script.onreadystatechange = null;
	                callback();
	            }
	        };
	    } else {  //Others
	        script.onload = function(){
	            callback();
	        };
	    }

	    script.src = url;
	    document.body.appendChild(script);
	}
	 
	/**
	 * Convierte un texto en minusculas y quita caracteres especiales
	 * @param {String}
	 * @returns {String}
	 */
	function pseudoSlugify(s){
		return s.toLowerCase().replace(/[áäà]/g,'a').replace(/[éëè]/g,'e').replace(/[íïì]/g,'i').replace(/[óöò]/g,'o').replace(/[úüù]/g,'u').replace(/ñ/g,'n');
	}

	/**
	 * Slugify
	 * @param {String}
	 * @returns {String}
	 */
	function slugify(s){
		return s.replace(/\s+/g,'-').replace(/[^a-z0-9\-]/g,'');	
	}

	/**
	 * Elimina saltos de línea, retornos de carro y tabulación
	 * @param {String}
	 * @returns {String}
	 */
	function stripNL(s){
		return s.replace(/[\n\r\t]/g,'');
	}
})(jQuery);
